@@ -5,12 +5,13 @@ from oauth2.web import Request, Response, SiteAdapter
 from oauth2.grant import ImplicitGrantHandler, AuthorizationCodeAuthHandler,\
     AuthRequestMixin, AuthorizationCodeTokenHandler, ImplicitGrant,\
     AuthorizationCodeGrant, ResourceOwnerGrantHandler, ResourceOwnerGrant,\
-    Scope, RefreshToken, RefreshTokenHandler, ScopeGrant
+    Scope, RefreshToken, RefreshTokenHandler, ScopeGrant,\
+    ClientCredentialsGrant, ClientCredentialsHandler
 from oauth2.store import ClientStore, AuthCodeStore, AccessTokenStore
 from oauth2.error import OAuthInvalidError, OAuthUserError, OAuthClientError,\
     ClientNotFoundError, UserNotAuthenticated, AccessTokenNotFound
-from oauth2 import AuthorizationController, Client, AuthorizationCode,\
-    AccessToken
+from oauth2 import Provider
+from oauth2.datatype import Client, AuthorizationCode, AccessToken
 from oauth2.tokengenerator import TokenGenerator
 
 def mock_time():
@@ -111,7 +112,7 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         request_mock.get_param.assert_has_calls([call("client_id"),
                                                 call("redirect_uri"),
                                                 call("state")])
-        scope_handler_mock.parse.assert_called_with(request_mock)
+        scope_handler_mock.parse.assert_called_with(request_mock, "query")
         clientStoreMock.fetch_by_client_id.assert_called_with(client_id)
         self.assertEqual(handler.client_id, client_id)
         self.assertEqual(handler.redirect_uri, redirect_uri)
@@ -224,7 +225,7 @@ class AuthRequestMixinTestCase(unittest.TestCase):
         request_mock.get_param.assert_has_calls([call("client_id"),
                                                 call("redirect_uri"),
                                                 call("state")])
-        scope_handler_mock.parse.assert_called_with(request_mock)
+        scope_handler_mock.parse.assert_called_with(request_mock, "query")
         clientStoreMock.fetch_by_client_id.assert_called_with(client_id)
         self.assertEqual(handler.client_id, client_id)
         self.assertEqual(handler.redirect_uri, redirect_uri)
@@ -1019,7 +1020,7 @@ class ResourceOwnerGrantTestCase(unittest.TestCase):
         site_adapter_mock = Mock(SiteAdapter)
         token_generator_mock = Mock()
         
-        server_mock = Mock(AuthorizationController)
+        server_mock = Mock(Provider)
         server_mock.access_token_store = access_token_store_mock
         server_mock.client_store = client_store_mock
         server_mock.site_adapter = site_adapter_mock
@@ -1036,7 +1037,7 @@ class ResourceOwnerGrantTestCase(unittest.TestCase):
         request_mock = Mock(Request)
         request_mock.post_param.return_value = "other"
         
-        server_mock = Mock(AuthorizationController)
+        server_mock = Mock(Provider)
         
         factory = ResourceOwnerGrant()
         
@@ -1201,7 +1202,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         
         client_store_mock.fetch_by_client_id.assert_called_with(client_id)
         scope_handler_mock.parse.assert_called_with(request=request_mock,
-                                                    source="POST")
+                                                    source="body")
         
         self.assertEqual(handler.client_id, client_id)
         self.assertEqual(handler.username, username)
@@ -1280,7 +1281,7 @@ class ResourceOwnerGrantHandlerTestCase(unittest.TestCase):
         self.assertEqual(error.explanation, "Could not authenticate client")
 
 class ScopeTestCase(unittest.TestCase):
-    def test_parse_scope_scope_present(self):
+    def test_parse_scope_scope_present_in_query(self):
         """
         Scope.parse should return a list of requested scopes
         """
@@ -1291,13 +1292,23 @@ class ScopeTestCase(unittest.TestCase):
         
         scope = Scope(available=["user_read", "friends_write", "friends_read"])
         
-        scope.parse(request=request_mock)
+        scope.parse(request=request_mock, source="query")
         
         request_mock.get_param.assert_called_with("scope")
         
         self.assertListEqual(expected_scopes, scope.scopes)
         self.assertFalse(scope.send_back)
     
+    def test_parse_scope_scope_present_in_body(self):
+        scope = Scope()
+        
+        request_mock = Mock(Request)
+        request_mock.post_param.return_value = None
+        
+        scope.parse(request=request_mock, source="body")
+        
+        request_mock.post_param.assert_called_with("scope")
+    
     def test_parse_scope_default_on_no_scope(self):
         """
         Scope.parse should return a list containing the default value if no scope present in request and default is set
@@ -1310,7 +1321,7 @@ class ScopeTestCase(unittest.TestCase):
         scope = Scope(available=["user_read", "friends_write", "friends_read"],
                       default="all")
         
-        scope.parse(request=request_mock)
+        scope.parse(request=request_mock, source="query")
         
         request_mock.get_param.assert_called_with("scope")
         
@@ -1329,7 +1340,7 @@ class ScopeTestCase(unittest.TestCase):
         scope = Scope(available=["user_read", "friends_write", "friends_read"],
                       default="all")
         
-        scope.parse(request=request_mock)
+        scope.parse(request=request_mock, source="query")
         
         request_mock.get_param.assert_called_with("scope")
         
@@ -1347,7 +1358,7 @@ class ScopeTestCase(unittest.TestCase):
         
         scope = Scope()
         
-        scope.parse(request=request_mock)
+        scope.parse(request=request_mock, source="query")
         
         request_mock.get_param.assert_called_with("scope")
         
@@ -1364,7 +1375,7 @@ class ScopeTestCase(unittest.TestCase):
         scope = Scope(available=["user_read", "friends_write", "friends_read"])
         
         with self.assertRaises(OAuthInvalidError) as expected:
-            scope.parse(request_mock)
+            scope.parse(request_mock, source="query")
         
         e = expected.exception
         
@@ -1424,7 +1435,7 @@ class RefreshTokenTestCase(unittest.TestCase):
         scope_handler_mock = Mock()
         token_generator_mock = Mock()
         
-        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock = Mock(spec=Provider)
         controller_mock.token_path = path
         controller_mock.access_token_store = access_token_store_mock
         controller_mock.client_store = client_store_mock
@@ -1453,7 +1464,7 @@ class RefreshTokenTestCase(unittest.TestCase):
         """
         RefreshToken should return 'None' if path in the request does not equal the token path
         """
-        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock = Mock(spec=Provider)
         controller_mock.token_path = "/token"
         
         request_mock = Mock(spec=Request)
@@ -1471,7 +1482,7 @@ class RefreshTokenTestCase(unittest.TestCase):
         """
         path = "/token"
         
-        controller_mock = Mock(spec=AuthorizationController)
+        controller_mock = Mock(spec=Provider)
         controller_mock.token_path = path
         
         request_mock = Mock(spec=Request)
@@ -1570,7 +1581,7 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
                                                   call("refresh_token")])
         access_token_store_mock.fetch_by_refresh_token.assert_called_with(refresh_token)
         client_store_mock.fetch_by_client_id.assert_called_with(client_id)
-        scope_handler_mock.parse.assert_called_with(request_mock)
+        scope_handler_mock.parse.assert_called_with(request_mock, "body")
         scope_handler_mock.compare.assert_called_with(scopes)
         
         self.assertEqual(handler.client_id, client_id)
@@ -1736,5 +1747,139 @@ class RefreshTokenHandlerTestCase(unittest.TestCase):
         self.assertEqual(e.error, "invalid_request")
         self.assertEqual(e.explanation, "Invalid refresh token")
 
+class ClientCredentialsGrantTestCase(unittest.TestCase):
+    def test_call(self):
+        token_path = "token"
+        
+        request_mock = Mock(spec=Request)
+        request_mock.path = token_path
+        request_mock.post_param.return_value = "client_credentials"
+        
+        access_token_store_mock = Mock()
+        client_store_mock = Mock()
+        token_generator_mock = Mock()
+        
+        scope_handler_mock = Mock(spec=Scope)
+        
+        server_mock = Mock()
+        server_mock.token_path = token_path
+        server_mock.access_token_store = access_token_store_mock
+        server_mock.client_store = client_store_mock
+        server_mock.scope_handler = scope_handler_mock
+        server_mock.token_generator = token_generator_mock
+        
+        grant = ClientCredentialsGrant()
+        handler = grant(request_mock, server_mock)
+        
+        request_mock.post_param.assert_called_with("grant_type")
+        self.assertTrue(isinstance(handler, ClientCredentialsHandler))
+        self.assertEqual(handler.access_token_store, access_token_store_mock)
+        self.assertEqual(handler.client_store, client_store_mock)
+        self.assertTrue(isinstance(handler.scope_handler, Scope))
+        self.assertEqual(handler.token_generator, token_generator_mock)
+    
+    def test_call_wrong_request_path(self):
+        request_mock = Mock(spec=Request)
+        request_mock.path = "authorize"
+        
+        server_mock = Mock()
+        server_mock.token_path = "token"
+        
+        grant = ClientCredentialsGrant()
+        handler = grant(request_mock, server_mock)
+        
+        self.assertEqual(handler, None)
+    
+    def test_call_other_grant_type(self):
+        token_path = "token"
+        
+        request_mock = Mock(spec=Request)
+        request_mock.path = token_path
+        request_mock.post_param.return_value = "other_grant"
+        
+        server_mock = Mock()
+        server_mock.token_path = token_path
+        
+        grant = ClientCredentialsGrant()
+        handler = grant(request_mock, server_mock)
+        
+        self.assertEqual(handler, None)
+
+class ClientCredentialsHandlerTestCase(unittest.TestCase):
+    @patch("time.time", mock_time)
+    def test_process(self):
+        client_id = "xyz"
+        expires_in = 600
+        token = "abcd"
+        scopes = ["foo", "bar"]
+        
+        expected_response_body = {"access_token": token,
+                                  "expires_in": expires_in,
+                                  "token_type": "Bearer",
+                                  "scope": scopes}
+        
+        access_token_store_mock = Mock(spec=AccessTokenStore)
+        
+        response_mock = Mock(spec=Response)
+        
+        scope_handler_mock = Mock(spec=Scope)
+        scope_handler_mock.send_back = True
+        scope_handler_mock.scopes = scopes
+        
+        token_generator_mock = Mock(spec=TokenGenerator)
+        token_generator_mock.generate.return_value = token
+        token_generator_mock.expires_in = expires_in
+        
+        handler = ClientCredentialsHandler(
+            access_token_store=access_token_store_mock,
+            client_store=Mock(),
+            scope_handler=scope_handler_mock,
+            token_generator=token_generator_mock)
+        handler.client_id = client_id
+        result_response = handler.process(request=Mock(),
+                                          response=response_mock, environ={})
+        
+        access_token, = access_token_store_mock.save_token.call_args[0]
+        self.assertTrue(isinstance(access_token, AccessToken))
+        self.assertEqual(access_token.client_id, client_id)
+        self.assertEqual(access_token.grant_type, "client_credentials")
+        self.assertEqual(access_token.token, token)
+        self.assertEqual(access_token.data, {})
+        self.assertEqual(access_token.expires_at, expires_in + 1000)
+        self.assertEqual(access_token.refresh_token, None)
+        self.assertEqual(access_token.scopes, scopes)
+        
+        response_mock.add_header.assert_called_with("Content-type",
+                                                    "application/json")
+        self.assertDictEqual(json.loads(result_response.body),
+                             expected_response_body)
+    
+    def test_read_validate_params(self):
+        client_id = "abc"
+        client_secret = "xyz"
+        
+        client_store_mock = Mock(spec=ClientStore)
+        client_store_mock.fetch_by_client_id.return_value = Client(
+            identifier=client_id,
+            secret=client_secret,
+            redirect_uris=[])
+        
+        scope_handler_mock = Mock(spec=Scope)
+        
+        request_mock = Mock(spec=Request)
+        request_mock.post_param.side_effect = [client_id, client_secret]
+        
+        handler = ClientCredentialsHandler(access_token_store=Mock(),
+                                           client_store=client_store_mock,
+                                           scope_handler=scope_handler_mock,
+                                           token_generator=Mock())
+        handler.read_validate_params(request_mock)
+        
+        client_store_mock.fetch_by_client_id.assert_called_with(client_id)
+        request_mock.post_param.assert_has_calls([call("client_id"),
+                                                  call("client_secret")])
+        scope_handler_mock.parse.assert_called_with(request=request_mock,
+                                                    source="body")
+
 if __name__ == "__main__":
     unittest.main()
